@@ -9,7 +9,7 @@ import copy  # TODO
 import logging
 import sys
 from collections import OrderedDict
-from typing import Union
+from typing import Union, Iterable
 import os
 
 import numpy as np
@@ -170,6 +170,9 @@ class TreeWalker:
             return True
         return False
 
+    def order_keys(self, keys: Iterable[str]):
+        return keys
+
     def walk(self, listener: TreeListener, tree: ast.Node) -> None:
         """
         Walks an AST tree recursively
@@ -182,7 +185,7 @@ class TreeWalker:
             getattr(listener, 'enterEvery')(tree)
         if hasattr(listener, 'enter' + name):
             getattr(listener, 'enter' + name)(tree)
-        for child_name in tree.__dict__.keys():
+        for child_name in self.order_keys(tree.__dict__.keys()):
             if self.skip_child(tree, child_name):
                 continue
             self.handle_walk(listener, tree.__dict__[child_name])
@@ -225,6 +228,9 @@ def flatten_extends(orig_class: Union[ast.Class, ast.InstanceClass], modificatio
     for extends in orig_class.extends:
         c = orig_class.find_class(extends.component, check_builtin_classes=True)
 
+        if str(c.full_reference()) == str(orig_class.full_reference()):
+            raise Exception("Cannot extend class '{}' with itself".format(c.full_reference()))
+
         if c.type == "__builtin":
             if len(orig_class.extends) > 1:
                 raise Exception(
@@ -300,7 +306,14 @@ def build_instance_tree(orig_class: Union[ast.Class, ast.InstanceClass], modific
         scope_class = class_mod_argument.scope if class_mod_argument.scope is not None else extended_orig_class
         argument = class_mod_argument.value
         if isinstance(argument, ast.ShortClassDefinition):
+            old_class = extended_orig_class.classes[argument.name]
             extended_orig_class.classes[argument.name] = scope_class.find_class(argument.component)
+
+            # Fix references to symbol types that were already in the instance tree
+            for sym_name, sym in extended_orig_class.symbols.items():
+                if isinstance(sym.type, ast.InstanceClass) and sym.type.name is old_class.name:
+                    c = extended_orig_class.classes[argument.name]
+                    sym.type = build_instance_tree(c, sym.class_modification, c.parent)
         elif isinstance(argument, ast.ComponentClause):
             # Redeclaration of symbols
             # TODO: Do we need to handle scoping of redeclarations of symbols?
@@ -673,7 +686,7 @@ class FunctionExpander(TreeListener):
     Listener to extract functions
     """
 
-    def __init__(self, node: ast.Tree, function_set: set):
+    def __init__(self, node: ast.Tree, function_set: OrderedDict):
         self.node = node
         self.function_set = function_set
         super().__init__()
@@ -873,8 +886,12 @@ def expand_connectors(node: ast.Class) -> None:
     for equation in orig_equations:
         if isinstance(equation, ast.ConnectClause):
             # expand connector
-            assert len(equation.left.child) == 0
-            assert len(equation.right.child) == 0
+            if len(equation.left.child) != 0:
+                raise Exception("Could not resolve {} in connect clause ({}*, {}*)".format(
+                    equation.left, equation.left, equation.right))
+            if len(equation.right.child) != 0:
+                raise Exception("Could not resolve {} in connect clause ({}*, {}*)".format(
+                    equation.right, equation.left, equation.right))
 
             sym_left = node.symbols[equation.left.name]
             sym_right = node.symbols[equation.right.name]
